# -*- python -*-
# $RCSfile: fileselector.py,v $
# $Revision: 1.18 $
# $Author: langer $
# $Date: 2005/05/13 18:43:50 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import debug
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO.GUI import oof_mainiteration
from oof2.common.IO.GUI import parameterwidgets
import gtk
import os
import weakref

class FileSelector(gtk.GtkFileSelection):
    def __init__(self, title='', filename=None, **kwargs):
        debug.mainthreadTest()
        gtk.GtkFileSelection.__init__(self, title=title)
        self.ok_button.connect("clicked", self.ok)
        self.cancel_button.connect("clicked", self.cancel)
        self.main_vbox.connect("destroy", self.cancel)
        self.selection_entry.connect('changed', self.entryCB)

        # If the constructor gets a 'params' argument, it will build a
        # ParameterTable into the file selector.  params must be a
        # list or tuple of Parameters.  The values of the Parameters
        # will be set after getFile() is called.
        try:
            params = kwargs['params']
        except KeyError:
            self.table = None
        else:
            self.table = parameterwidgets.ParameterTable(params)
            self.main_vbox.add(self.table.gtk)

        self.set_modal(1)
        if not filename is None:
            self.set_filename(filename)
        self.got_file = 0
        self.sensitize()
    def entryCB(self, gtkobj):
        self.sensitize()
    def sensitize(self):
        self.ok_button.set_sensitive(self.selection_entry.get_text() != '')
    def file(self):
        return self.filename
    def ok(self, button):
        self.got_file = 1
        self.done = 1
    def cancel(self, button):
        self.got_file = 0
        self.done = 1
        self.cancelled = 1
    def getFile(self):
        self.done = 0
        self.cancelled = 0
        self.show()
        while not self.done:
            oof_mainiteration.mainiteration()
        if self.table:
            try:
                self.table.get_values()
            except parameter.ParameterMismatch:
                # Ignore exceptions only if the transaction was cancelled.
                self.got_file = 0
                if not self.cancelled:
                    raise
        self.hide()
        if self.got_file:
            return self.get_filename()
        else:
            return None


# This function should be called before you write to a file.
# If the file exists, it will prompt the user, asking whether to
# over-write or append.  It will return either 'w' for writing,
# 'a' for appending, or None, in which case the caller should not
# write to the file at all.
def get_file_mode(filename, no_append=0):
    mode = None
    try:
        os.stat(filename)               # does file exist?
    except OSError:                     # file not found
        mode = 'w'
    else:
        if no_append:
            ans = reporter.query("Overwrite file\n%s?" % filename,
            "OK", "Cancel", default="OK")
            if ans == "OK":
                mode = 'w'
        else:
            ans = reporter.query("Overwrite file\n%s?" % filename,
                "OK", "Append", "Cancel", default="OK")
            if ans == 'OK':
                mode = 'w'
            elif ans == 'Append':
                mode = 'a'
    return mode

# The _selectors dictionary keeps track of file selectors to be
# reused.  They are reused so that they preserve their state (current
# directory, etc).  The dictionary is a WeakValueDictionary because
# the selectors can be destroyed by the window manager, and we don't
# have any control over that.

_selectors = weakref.WeakValueDictionary()


# Always check the return value of fileSelector()!  It returns None if
# the user cancels the operation.

def fileSelector(**kwargs):
    try:
        widget = kwargs['widget']
    except KeyError:
        widget = FileSelector

    ident = None
    try:
        ident = kwargs['ident']
        selector = _selectors[ident]    # reuse old selector
    except (KeyError, TypeError):
        # Have to catch TypeError here, because that's what the Python
        # 2.1 WeakValueDictionary throws when you try to retrieve a
        # value using a key which is not present in the dictionary.
        # This is Python 2.1 broken-ness.
        selector = widget(**kwargs)     # create new selector
        if ident is not None:
            _selectors[ident] = selector
        
    try:
        selector.set_title(kwargs['title'])
    except KeyError:
        pass

    file = selector.getFile()

    if not kwargs.has_key('ident'):
        selector.destroy()
    return file
            
