# -*- python -*-
# $RCSfile: pixelselecttoolboxGUI.py,v $
# $Revision: 1.12 $
# $Author: langer $
# $Date: 2005/05/13 18:43:59 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# Pixel selection tools.  See comments in common/pixelselect.py.

from oof2.SWIG.common import switchboard
from oof2.SWIG.common.IO.GUI import rubberband
from oof2.common import debug
from oof2.common import pixelselectionmethod
from oof2.common import primitives
from oof2.common.IO import pixelselectiontoolbox
from oof2.common.IO.GUI import genericselectGUI

class PixelSelectToolboxGUI(genericselectGUI.GenericSelectToolboxGUI):
    def __init__(self, pixelselecttoolbox, method):
        debug.mainthreadTest()
        genericselectGUI.GenericSelectToolboxGUI.__init__(self,
                                                   "Pixel Selection",
                                                   pixelselecttoolbox,
                                                   method)

        # Switchboard callbacks that should be performed even when the
        # toolbox isn't active go here.  Callbacks that are performed
        # only when the toolbox IS active are installed in activate().
        self.sbcallbacks.append(
            switchboard.requestCallback('new pixel selection',
                                        self.newSelection)
            )

    def activate(self):
        genericselectGUI.GenericSelectToolboxGUI.activate(self)
        self.activecallbacks = [
            switchboard.requestCallback((self.gfxwindow(),
                                         'layers changed'),
                                        self.layerChangeCB),
            switchboard.requestCallback('pixel selection changed',
                                        self.changedSelection)
            ]

    def deactivate(self):
        genericselectGUI.GenericSelectToolboxGUI.deactivate(self)
        map(switchboard.removeCallback, self.activecallbacks)
        self.activecallbacks = []

    def getSource(self):
        top= self.gfxwindow().topwho('Microstructure', 'Image')
        return self.gfxwindow().topwho('Microstructure', 'Image')
    
    def finish_up(self, ptlist, shift, ctrl, selmeth):
        # copy parameters from widgets to the registration
        self.selectionMethodFactory.set_defaults()
        
        # Now invoke the selection method by calling the
        # corresponding toolbox menu item. The arguments are
        # the Registration parameters, plus the list of Points
        # and the keyboard modifiers.
        menuitem = getattr(self.toolbox.menu, selmeth.name())
        menuitem.callWithDefaults(selectable=self.getSourceName(),
                                  points=ptlist, shift=shift, ctrl=ctrl)

    def undoCB(self, button):
        self.toolbox.menu.Undo(selectable=self.getSourceName())

    def redoCB(self, button):
        self.toolbox.menu.Redo(selectable=self.getSourceName())

    def clearCB(self, button):
        self.toolbox.menu.Clear(selectable=self.getSourceName())

    def invertCB(self, button):
        self.toolbox.menu.Invert(selectable=self.getSourceName())



#######################################

# Redefine PixelSelectToolbox.makeGUI() so that it returns a
# PixelSelectToolboxGUI object.

def _makeGUI(self):
    return PixelSelectToolboxGUI(self, self.method)

pixelselectiontoolbox.PixelSelectToolbox.makeGUI = _makeGUI


######################

## Assign rubberband creation functions to the
## PixelSelectionRegistrations.  Most assignments are to *instances*,
## and as such are not member functions.  The default assignment (no
## rubberband) is to the class, and so the function needs a 'self'
## argument.

def _NoRubberBand(self, reg):
    return rubberband.NoRubberBand()
pixelselectionmethod.PixelSelectionRegistration.getRubberBand = _NoRubberBand


def _BrushSelectorRB(reg):
    style = reg.getParameter('style').value
    return rubberband.BrushRubberBand(style)

pixelselectionmethod.brushSelectorRegistration.getRubberBand = _BrushSelectorRB


def _RectangleSelectorRB(reg):
    return rubberband.RectangleRubberBand()

pixelselectionmethod.rectangleSelectorRegistration.getRubberBand = \
                                                     _RectangleSelectorRB


def _CircleSelectorRB(reg):
    return rubberband.CircleRubberBand()

pixelselectionmethod.circleSelectorRegistration.getRubberBand = \
                                                              _CircleSelectorRB


def _EllipseSelectorRB(reg):
    return rubberband.EllipseRubberBand()

pixelselectionmethod.ellipseSelectorRegistration.getRubberBand = \
                                                             _EllipseSelectorRB
