# -*- python -*-
# $RCSfile: viewertoolboxGUI.py,v $
# $Revision: 1.12 $
# $Author: langer $
# $Date: 2005/06/08 19:59:59 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import ooferror
from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import primitives
from oof2.common.IO import reporter
from oof2.common.IO import viewertoolbox
from oof2.common.IO.mainmenu import OOF
from oof2.common.IO.GUI import mousehandler
from oof2.common.IO.GUI import toolboxGUI
from gtk import *

class ViewerToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, viewertoolbox):
        debug.mainthreadTest()

        toolboxGUI.GfxToolbox.__init__(self, "Viewer", viewertoolbox)
        mainbox = GtkVBox()
        self.gtk.add(mainbox)

        infoframe = GtkFrame("Position Information")
        infotable = GtkTable(cols=3, rows=2)
        pixellabel = GtkLabel("Pixel: ")
        self.pixel_x = GtkEntry()
        self.pixel_x.set_usize(25, -1)
        self.pixel_x.set_editable(0)
        self.pixel_y = GtkEntry()
        self.pixel_y.set_usize(25, -1)
        self.pixel_y.set_editable(0)
        physicallabel = GtkLabel("Physical: ")
        self.physical_x = GtkEntry()
        self.physical_x.set_usize(80, -1)
        self.physical_x.set_editable(0)
        self.physical_y = GtkEntry()
        self.physical_y.set_usize(80, -1)
        self.physical_y.set_editable(0)
        infotable.attach(pixellabel, 0,1, 0,1, xpadding=5, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.pixel_x, 1,2, 0,1, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.pixel_y, 2,3, 0,1, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(physicallabel, 0,1, 1,2, xpadding=5, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.physical_x, 1,2, 1,2, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.physical_y, 2,3, 1,2, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infoframe.add(infotable)
        mainbox.add(infoframe, fill=0, expand=0)

        zoomframe = GtkFrame("Zoom")
        mainbox.add(zoomframe, fill=0, expand=0)
        zoombox = GtkVBox()
        zoomframe.add(zoombox)

        buttonrow = GtkHBox(homogeneous=1, spacing=2)
        zoombox.add(buttonrow, expand=0, fill=1, padding=2)
        inbutton = GtkButton("In")
        buttonrow.add(inbutton, expand=0, fill=1)
        inbutton.connect('clicked', self.inCB)
        outbutton = GtkButton("Out")
        buttonrow.add(outbutton, expand=0, fill=1)
        outbutton.connect('clicked', self.outCB)
        fillbutton = GtkButton("Fill Window")
        buttonrow.add(fillbutton, expand=0, fill=1)
        fillbutton.connect('clicked', self.fillCB)

        factorrow = GtkHBox()
        zoombox.add(factorrow, expand=0, fill=0, padding=2)
        factorrow.add(GtkLabel("Factor: "))
        self.zoomfactor = GtkEntry()
        self.zoomfactor.set_editable(1)
        self.zoomfactor.set_usize(137, -1)
        factorrow.add(self.zoomfactor, expand=1, fill=1)

        label0 = GtkLabel("Shift+Click: Zoom in\nCtrl+Click: Zoom out")
        label0.set_pattern("             _______\n            ________\n")
        label0.set_justify(JUSTIFY_LEFT)
        zoombox.add(GtkHSeparator(), fill=0, expand=0, padding=2)
        zoombox.add(label0, fill=0, expand=0)

        infobox = GtkHButtonBox()
        canvas_info = GtkButton("Canvas Info")
        canvas_info.connect("clicked", self.canvas_infoCB)
        guitop.top().tooltip(canvas_info,
                           "Display canvas information in the message window.")
        infobox.add(canvas_info)
        mainbox.add(infobox)

        self.currentZFactor = self.gfxwindow().zoomFactor()

        switchboard.requestCallback("zoom factor changed", self.zfactorCB)

        # Make sure that the Zoom commands in the graphics windows
        # Settings menu use an up-to-date zoom factor.
        self.gfxwindow().menu.Settings.Zoom.In.add_gui_callback(
            self.wrapMenuZoom)
        self.gfxwindow().menu.Settings.Zoom.Out.add_gui_callback(
            self.wrapMenuZoom)

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.updateZFactor()

    def zfactorCB(self, *args):         # switchboard "zoom factor changed"
        self.updateZFactor()
        
    def updateZFactor(self):
        self.currentZFactor = self.gfxwindow().zoomFactor()
        self.zoomfactor.set_text("%g"% self.currentZFactor)

    def zoomfactorChanged(self, *args):  # gtk callback
        self.getNewZoomFactor()

    def wrapMenuZoom(self, menuitem):
        # gui callback for Zoom commands in the Settings menu.
        self.getNewZoomFactor()
        menuitem()
        
    def findMicrostructure(self):
        who = self.toolbox.gfxwindow().topwho('Microstructure', 'Image',
                                              'Skeleton', 'Mesh')
        if who is not None:
            return who.getMicrostructure()

    def acceptEvent(self, eventtype):
        return eventtype in ('down', 'move', 'up')

    # World coord from world_coord() needs to be recalibrated.
    def recalibrate(self, point):
        return primitives.Point(point.x, -point.y)

    def get_proper_world_coord(self, i, j):
        temp = self.gfxwindow().oofcanvas.world_coord(i, j)
        return self.recalibrate(temp)

    def canvas_infoCB(self, *args):
        # Visible canvas size in pixel units
        xmax = self.gfxwindow().oofcanvas.get_width_in_pixels()-1
        ymax = self.gfxwindow().oofcanvas.get_height_in_pixels()-1
        # Visible MS in physical units
        ll = self.get_proper_world_coord(0, ymax)
        ur = self.get_proper_world_coord(xmax, 0)
        reporter.report("### Canvas Information ###")
        reporter.report("Lower-left corner : ", ll)
        reporter.report("Upper-right corner: ", ur)
        reporter.report("H-scroll position : ",
                        self.gfxwindow().oofcanvas.hScrollPosition())
        reporter.report("V-scroll position : ",
                        self.gfxwindow().oofcanvas.vScrollPosition())
        reporter.report("Pixels per unit   : ",
                        self.gfxwindow().oofcanvas.get_pixels_per_unit())

    def inCB(self, *args):              # gtk callback
        self.getNewZoomFactor()
        self.gfxwindow().menu.Settings.Zoom.In()
    def outCB(self, *args):             # gtk callback
        self.getNewZoomFactor()
        self.gfxwindow().menu.Settings.Zoom.Out()
    def fillCB(self, *args):            # gtk callback
        self.gfxwindow().menu.Settings.Zoom.Fill_Window()

    def getNewZoomFactor(self):
        # The zoomfactor widget doesn't have any gtk callbacks because
        # all of the signals generate either too few or too many
        # events.  getNewZoomFactor must be called before the zoom
        # factor is used in case the user has changed the zoomfactor
        # widget.
        txt = self.zoomfactor.get_text()
        if not txt:
            # Nothing's been entered yet, just use the default value
            return
        try:
            factor = float(txt)
        except ValueError:
            raise ooferror.ErrUserError("Bad zoom factor! %s" % txt)
        if factor != self.currentZFactor:
            # Use runBlock to ensure that the new zoom factor is set
            # before being used.
            mainthread.runBlock(
                self.gfxwindow().menu.Settings.Zoom.Zoom_Factor,
                kwargs={'factor':factor})
        
    def up(self, x, y, shift, ctrl):
        debug.mainthreadTest()
        ms = self.findMicrostructure()
        if ms:
            pixel = ms.pixelFromPoint(primitives.Point(x,y))
            self.pixel_x.set_text("%d" % pixel.x)
            self.pixel_y.set_text("%d" % pixel.y)
            self.physical_x.set_text("%-11.4g" % x)
            self.physical_y.set_text("%-11.4g" % y)

        if shift and not ctrl:
            self.getNewZoomFactor()
            self.gfxwindow().menu.Settings.Zoom.InFocussed(
                focus=primitives.Point(x,y))
        elif not shift and ctrl:
            self.getNewZoomFactor()
            self.gfxwindow().menu.Settings.Zoom.OutFocussed(
                focus=primitives.Point(x,y))

    def move(self, x, y, shift, ctrl):
        debug.mainthreadTest()

def _makeGUI(self):
    return ViewerToolboxGUI(self)

viewertoolbox.ViewerToolbox.makeGUI = _makeGUI
    
