// -*- C++ -*-
// $RCSfile: bitoverlay.C,v $
// $Revision: 1.16 $
// $Author: langer $
// $Date: 2004/05/07 18:57:41 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "common/IO/bitoverlay.h"
#include "common/trace.h"
#include "common/tostring.h"
#include "common/IO/stringimage.h"

BitmapOverlay::BitmapOverlay(const Coord *size, const ICoord *isize)
  : fg(CColor(1., 1., 1.)),
    bg(CColor(0., 0., 0.)),
    stringImage_(0),
    stringUpToDate_(false)
{
    resize(size, isize);
}

BitmapOverlay::~BitmapOverlay() {
  if(stringImage_)
    delete stringImage_;
}

void BitmapOverlay::resize(const Coord *size, const ICoord *isize) {
  size_ = *size;
  sizeInPixels_ = *isize;
  clear();
}

void BitmapOverlay::clear() {
  data.resize(sizeInPixels_);
  data.clear(false);
  ++timestamp;
  stringUpToDate_ = false;
}

void BitmapOverlay::invert() {
  data.invert();
}

void BitmapOverlay::set(const ICoord *pixel) {
  data.set(pixel);
  ++timestamp;
  stringUpToDate_ = false;
}

void BitmapOverlay::set(const std::vector<ICoord> *pixels) {
  for(std::vector<ICoord>::const_iterator i=pixels->begin();i<pixels->end();++i)
    if(data.contains(*i))
      data.set(&*i);
  ++timestamp;
  stringUpToDate_ = false;
}

void BitmapOverlay::reset(const std::vector<ICoord> *pixels) {
  for(std::vector<ICoord>::const_iterator i=pixels->begin();i<pixels->end();++i)
    if(data.contains(*i))
      data.reset(&*i);
  ++timestamp;
  stringUpToDate_ = false;
}

void BitmapOverlay::reset(const ICoord *pixel) {
  data.reset(pixel);
  ++timestamp;
  stringUpToDate_ = false;
}

void BitmapOverlay::toggle(const ICoord *pixel) {
  data.toggle(pixel);
  ++timestamp;
  stringUpToDate_ = false;
}

void BitmapOverlay::toggle(const std::vector<ICoord> *pixels) {
  for(std::vector<ICoord>::const_iterator i=pixels->begin();i<pixels->end();++i)
    if(data.contains(*i))
      data.toggle(&*i);
  ++timestamp;
  stringUpToDate_ = false;
}

bool BitmapOverlay::get(const ICoord *pixel) const {
  if(data.contains(*pixel))
    return data.get(*pixel);
  return false;
}

void BitmapOverlay::copy(const BitmapOverlay *other) {
  data = other->data.clone();
}

// std::vector<ICoord> *BitmapOverlay::pixels(int v) const {
//   std::vector<ICoord> *pxls = new std::vector<ICoord>;
//   for(Array<bool>::const_iterator i=data.begin(); i!=data.end(); ++i)
//     if(*i == v)
//       pxls->push_back(i.coord());
//   return pxls;
// }

void BitmapOverlay::setColor(const CColor *color) {
  fg = *color;
  if(fg.getRed() == 0.0) {
    bg.setRed(1.0);
    bg.setGreen(1.0);
    bg.setBlue(1.0);
  }
  else {
    bg.setRed(0.0); 
    bg.setBlue(0.0);
    bg.setGreen(0.0);
  }
  stringUpToDate_ = false;
}

CColor BitmapOverlay::getBG() const {
  return bg;
}

CColor BitmapOverlay::getFG() const {
  return fg;
}

// Construct a string representation of the image, for making a
// GdkImlibImage.

StringImage *BitmapOverlay::stringimage() const {
  if(stringImage_) {		// string image is already computed
    if(stringUpToDate_)		// ... and it's usable
      return stringImage_;
    delete stringImage_;	// ... it's not usable
  }
  stringImage_ = new StringImage(&sizeInPixels_, &size_);
  //  Set each pixel in the string image
  for(Array<bool>::const_iterator i=data.begin(); i!=data.end(); ++i) {
    if(data[i])
      stringImage_->set(&i.coord(), &fg);
    else
      stringImage_->set(&i.coord(), &bg);
  }
  stringUpToDate_ = true;
  return stringImage_;
}
