# -*- python -*-
# $RCSfile: material_.py,v $
# $Revision: 1.36 $
# $Author: langer $
# $Date: 2005/05/13 18:44:05 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common.IO import oofmenu
import string, copy

# Set of all materials
material_library = {}

def newCMaterial(name, *properties):
    try:
        material = material_library[name]
    except KeyError:
        material = Material(name)
    else:
        # Material already existed. Remove all of its properties.
        for property in material.properties():
            material.removeProperty(property)

    # Assign properties.
    for prop in properties:
        material.addProperty(prop)

    return material

#
# Extensions to the SWIG'd class:

def _Material___init__(self, name):
    self.this = materialc.new_Material(name)
    self.thisown = 1
    material_library[name] = self
    # addProperty and removeProperty maintain a Python list of
    # property instances, which ensures that there exists a Python
    # reference to each property in a material.  Reference-count
    # manipulations are not required in the C++ code.
    self.proplist = []

Material.__init__ = _Material___init__

_old__del__ = Material.__del__
def _Material___del__(self):
    switchboard.removeCallback(self.sbcallback)
    _old__del__(self)

Material.__del__ = _Material___del__

def _Material_addProperty(self, prop):
    self.add1Property(prop)
    self.proplist.append(prop)
    self.cross_reference()

Material.addProperty = _Material_addProperty
    
def _Material_removeProperty_(self, *properties):
    for prop in properties:
        # Exception thrown by C++ code if the property is not present.
        self.remove1Property(prop)  # remove from C++ list
        self.proplist.remove(prop)
    self.cross_reference()

Material.removeProperty = _Material_removeProperty_

def _Material_properties(self):
    return [self.getProperty(i) for i in range(self.nProperties())]

MaterialPtr.properties = _Material_properties

# Return true if we have an active nonlinear property.
def _Material_nonlinear(self,femesh):
    for p in self.properties():
        if p.is_active(femesh):
            # If property is active, check the registration for the flag.
            r = p.registration()
            if r.nonlinear():
                return 1
    return 0


MaterialPtr.nonlinear = _Material_nonlinear

######

# A Material's identity is tied to its name.  Python object identity
# doesn't work, because the same Material returned to Python via SWIG
# gets wrapped in different objects each time it's returned.

def _Material__eq__(self, other):
    return isinstance(other, MaterialPtr) and other.name() == self.name()
MaterialPtr.__eq__ = _Material__eq__

def _Material__ne__(self, other):
    return not self.__eq__(other)
MaterialPtr.__ne__ = _Material__ne__

def _Material__hash__(self):
    return hash(self.name())
MaterialPtr.__hash__ = _Material__hash__

######

def _Material___repr_(self):
    out = ["Material('" + self.name() + "'"]
    for prop in self.properties():
        out.append(`prop`)
    out = string.join(out, ",\n\t")
    return out + ")"

MaterialPtr.__repr__ = _Material___repr_

def _Material_copy_(self, newname):
    args = [newname] +  [copy.deepcopy(p) for p in self.properties()]
    m = newCMaterial(*args)
    return m

MaterialPtr.copy = _Material_copy_

##############################

# Create the singleton material attribute registration object.
# The (swig'd) constructor adds it to the global list of attributes.
attributeReg = MaterialAttributeRegistration()

###################

_old_getMaterials = getMaterials
def getMaterials(microstructure):
    return map(MaterialPtr, _old_getMaterials(microstructure))

