// -*- C++ -*-
// $RCSfile: ooferror.h,v $
// $Revision: 1.2 $
// $Author: reida $
// $Date: 2004/03/23 18:43:01 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef ENGINE_OOFERROR_H
#define ENGINE_OOFERROR_H

#include "common/ooferror.h"

class ErrNoSuchField : public ErrUserError {
private:
  const std::string field;
public:
  ErrNoSuchField(const std::string &field)
    : ErrUserError("No such field: " + field),
      field(field)
  {}
  virtual ~ErrNoSuchField() {}
  virtual std::string pythonequiv() const;

};

class ErrDuplicateField : public ErrUserError {
private:
  const std::string field;
  const std::string oldtype;
  const std::string newtype;
public:
  ErrDuplicateField(const std::string &field,
		    const std::string &newtype,
		    const std::string &oldtype)
    : ErrUserError("New field " + field + " of type " + newtype + " conflicts with old field of type " + oldtype),
      field(field),
      oldtype(oldtype),
      newtype(newtype)
  {}
  virtual ~ErrDuplicateField() {}
  virtual std::string pythonequiv() const;

};

// Trying to get a property that a Material doesn't have.

class ErrNoSuchProperty : public ErrUserError {
private:
  const std::string material;
public:
  ErrNoSuchProperty(const std::string &mat, const std::string &prop);
  const std::string propname;
  virtual std::string pythonequiv() const;
};

class ErrPropertyMissing : public ErrUserError {
private:
  const std::string material;
  const std::string propname;
  const std::string missingprop;
public:
  ErrPropertyMissing(const std::string &mat, const std::string &prop,
		     const std::string &miss)
    : ErrUserError("Property \"" + prop + "\" in Material \"" + mat +
		   "\" requires a Property of class \"" + miss + "\""),
    material(mat),
    propname(prop),
    missingprop(miss)
  {}
  virtual std::string pythonequiv() const {
    return
      "ErrPropertyMissing('"+material+"','"+propname +"','"+missingprop+"')";
  }
};

class ErrRedundantProperty : public ErrUserError {
private:
  std::string tag;
public:
  ErrRedundantProperty(const std::string &t)
    : tag(t), ErrUserError("")
  {}
  const std::string &get_tag() { return tag; }
};


#endif // ENGINE_OOFERROR_H
