// -*- C++ -*-
// $RCSfile: aniso.h,v $
// $Revision: 1.9 $
// $Author: redwing $
// $Date: 2004/08/31 19:07:04 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef ANISOELASTICITY_H
#define ANISOELASTICITY_H

#include "engine/property/elasticity/homogeneous.h"
#include "engine/property/elasticity/cijkl.h"
#include "common/pythonexportable.h"
#include <string>

class OrientationProp;
class EulerAngle;

class AnisoElasticity
  : public HomogeneousElasticity, virtual public PythonNative<Property>
{
public:
  AnisoElasticity(PyObject *registry, PyObject *self, 
		  const std::string &nm, const Cijkl &c)
    : HomogeneousElasticity(registry,nm),
      PythonNative<Property>(self),
      crystal_cijkl_(c)
  {}
  virtual ~AnisoElasticity() {
    // std::cerr << "Destroying AnisoElasticity" << std::endl;
  }
  virtual const Cijkl cijkl() const;
  virtual const Cijkl &crystal_cijkl() const;
private:
  const OrientationProp *orientation;
  virtual void precompute();
  virtual void cross_reference(Material*);
  Cijkl crystal_cijkl_;
  Cijkl lab_cijkl;
};


#endif // ANISOELASTICITY_H
