# -*- python -*-
# $RCSfile: permittivity_.py,v $
# $Revision: 1.8 $
# $Author: langer $
# $Date: 2005/05/13 18:44:18 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common import utils
from oof2.common.IO import parameter
from oof2.engine import problem
from oof2.engine import propertyregistration
from oof2.SWIG.engine import symmmatrix

PropertyRegistration = propertyregistration.PropertyRegistration

dielectricConductivity = PropertyRegistration(
    'Electric:DielectricPermittivity:Isotropic',
    IsoDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",
    ordering=98,
    params=[parameter.FloatParameter('epsilon',
                                     1.0, tip=parameter.emptyTipString)],
    fields=[problem.Voltage],
    fluxes=[problem.Total_Polarization],
    outputs=["Energy"],
    propertyType="DielectricPermittivity",
    tip="Isotropic dielectric permittivity."
    )

# Reprs refer to the registered name, not the actual class name,
# because properties are instanced via the registry.
def _IsoDielectricPermittivity___repr__(self):
    return "Isotropic(name='%s', epsilon=%s)" % \
           (self.name(), `self.epsilon()` )

IsoDielectricPermittivityPtr.__repr__ = _IsoDielectricPermittivity___repr__



## TRICLINIC

class TriclinicDielectricPermittivity(AnisoDielectricPermittivity):
    def __init__(self, registration, name, kij):
        AnisoDielectricPermittivity.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Triclinic(name='%s', epsilon=%s)" % \
               (self.name(),`self.epsilon()`)


PropertyRegistration(
    'Electric:DielectricPermittivity:Anisotropic:Triclinic', TriclinicDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",105,
    [symmmatrix.TriclinicRank2TensorParameter(
    'epsilon',
    symmmatrix.TriclinicRank2Tensor(xx=1.0, yy=1.0, zz=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Voltage], fluxes=[problem.Total_Polarization],
    propertyType="DielectricPermittivity",
    tip="Triclinic dielectric permittivity")

   

## MONOCLINIC

class MonoclinicDielectricPermittivity(AnisoDielectricPermittivity):
    def __init__(self, registration, name, kij):
        AnisoDielectricPermittivity.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Monoclinic(name='%s', epsilon=%s)" % \
               (self.name(),`self.epsilon()`)


PropertyRegistration(
    'Electric:DielectricPermittivity:Anisotropic:Monoclinic', MonoclinicDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",104,
    [symmmatrix.MonoclinicRank2TensorParameter(
    'epsilon',
    symmmatrix.MonoclinicRank2Tensor(xx=1.0, yy=1.0, zz=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Voltage], fluxes=[problem.Total_Polarization],
    propertyType="DielectricPermittivity",
    tip="Monoclinic dielectric permittivity.")


## ORTHORHOMBIC
class OrthorhombicDielectricPermittivity(AnisoDielectricPermittivity):
    def __init__(self, registration, name, kij):
        AnisoDielectricPermittivity.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Orthorhombic(name='%s', epsilon=%s)" % \
               (self.name(),`self.epsilon()`)


PropertyRegistration(
    'Electric:DielectricPermittivity:Anisotropic:Orthorhombic', OrthorhombicDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",103,
    [symmmatrix.OrthorhombicRank2TensorParameter(
    'epsilon',
    symmmatrix.OrthorhombicRank2Tensor(xx=1.0, yy=1.0, zz=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Voltage], fluxes=[problem.Total_Polarization],
    propertyType="DielectricPermittivity",
    tip="Orthorhombic dielectric permittivity.")

## For the following crystal structures: tetragonal, trigonal, and hexagonal,
## eventhough the registration is different, the GUI and registration
## parameter objects are the same.

## TETRAGONAL
class TetragonalDielectricPermittivity(AnisoDielectricPermittivity):
    def __init__(self, registration, name, kij):
        AnisoDielectricPermittivity.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Tetragonal(name='%s', epsilon=%s)" % \
               (self.name(),`self.epsilon()`)


PropertyRegistration(
    'Electric:DielectricPermittivity:Anisotropic:Tetragonal', TetragonalDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",101,
    [symmmatrix.TetragonalRank2TensorParameter(
    'epsilon',
    symmmatrix.TetragonalRank2Tensor(xx=1.0, zz=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Voltage], fluxes=[problem.Total_Polarization],
    propertyType="DielectricPermittivity",
    tip="Tetragonal dielectric permittivity.")

## TRIGONAL
class TrigonalDielectricPermittivity(AnisoDielectricPermittivity):
    def __init__(self, registration, name, kij):
        AnisoDielectricPermittivity.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Trigonal(name='%s', epsilon=%s)" % \
               (self.name(),`self.epsilon()`)


PropertyRegistration(
    'Electric:DielectricPermittivity:Anisotropic:Trigonal', TrigonalDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",102,
    [symmmatrix.TrigonalRank2TensorParameter(
    'epsilon',
    symmmatrix.TrigonalRank2Tensor(xx=1.0, zz=0.5),
    tip=parameter.emptyTipString)],
    fields=[problem.Voltage], fluxes=[problem.Total_Polarization],
    propertyType="DielectricPermittivity",
    tip="Trigonal dielectric permittivity")

## HEXAGONAL
class HexagonalDielectricPermittivity(AnisoDielectricPermittivity):
    def __init__(self, registration, name, kij):
        AnisoDielectricPermittivity.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Hexagonal(name='%s', epsilon=%s)" % \
               (self.name(),`self.epsilon()`)


PropertyRegistration(
    'Electric:DielectricPermittivity:Anisotropic:Hexagonal', HexagonalDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",100,
    [symmmatrix.HexagonalRank2TensorParameter(
    'epsilon',
    symmmatrix.HexagonalRank2Tensor(xx=1.0, zz=0.5),
    tip=parameter.emptyTipString)],
    fields=[problem.Voltage], fluxes=[problem.Total_Polarization],
    propertyType="DielectricPermittivity",
    tip="Hexagonal dielectric permittivity")


## CUBIC
class CubicDielectricPermittivity(AnisoDielectricPermittivity):
    def __init__(self, registration, name, kij):
        AnisoDielectricPermittivity.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Cubic(name='%s', epsilon=%s)" % \
               (self.name(),`self.epsilon()`)


PropertyRegistration(
    'Electric:DielectricPermittivity:Anisotropic:Cubic', TetragonalDielectricPermittivity,
    "oof2.SWIG.engine.property.permittivity.permittivity",99,
    [symmmatrix.CubicRank2TensorParameter(
    'epsilon',
    symmmatrix.CubicRank2Tensor(xx=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Voltage], fluxes=[problem.Total_Polarization],
    propertyType="DielectricPermittivity",
    tip="Cubic dielectric permittivity")


## *Electric* BODY forces (sorry Andrew... --REG)

PropertyRegistration(
    'Electric:SpaceCharge', ChargeDensity,
    "oof2.SWIG.engine.property.permittivity.permittivity",150,
    [parameter.FloatParameter('charge', 0.0, tip="electrostatic charge density")],
    fluxes=[problem.Total_Polarization],
    propertyType="ChargeDensity",
    tip="Spatial charge density")

def _ChargeDensity___repr__(self):
    return "ChargeDensity(name='%s', chargedensity=%s)" % (self.name(), `self.rho()`)

ChargeDensityPtr.__repr__ = _ChargeDensity___repr__
