// -*- C++ -*-
// $RCSfile: stiffnessmatrix.h,v $
// $Revision: 1.9 $
// $Author: reida $
// $Date: 2004/09/08 15:15:08 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef STIFFNESSMATRIX_H
#define STIFFNESSMATRIX_H

#include <vector>
#include <iostream>
#include "sparselink.h"
#include "engine/matvec.h"

class CSRmatrix;
class FEMesh;

class StiffnessMatrix : public SparseLinkMatrix<double> {
private:
  FEMesh *mesh;
  std::vector<int> subgroup(const std::vector<int> group, 
			    const std::vector<int> &map) const;
  //  VECTOR_I rowgroup, colgroup;
public:
  StiffnessMatrix(FEMesh*);
  StiffnessMatrix(FEMesh*, int, int);
  // The rowgroup and colgroup vectors are filled in during the
  // stiffness-matrix construction phase (by equation.C).  For each
  // row or column of the matrix, the corresponding vector value is -1
  // if the DOF or equation is in-plane, and otherwise is equal to the
  // node number.  This info is used by the block preconditioner to
  // group matrix elements by nodes.
  std::vector<int> rowgroup, colgroup;
  
  // NB order of arguments is important here, getting them
  // backwards is wrong but will not generate an error.
  CSRmatrix submatrix(const std::vector<int> &rowmap,
 		      const std::vector<int> &colmap) const;
  // These functions use the appropriate maps to make sublists of the
  // rowgroup and colgroup vectors analogous to the way the submatrix
  // function works.
  std::vector<int> rowgroups(const std::vector<int> &rowmap) const;
  std::vector<int> colgroups(const std::vector<int> &colmap) const;
};

#endif // STIFFNESSMATRIX_H
